@@ -225,8 +225,8 @@ define_instruction! {
     /// and <https://en.wikipedia.org/wiki/CHIP-8#Opcode_table> (CC-BY-SA 3.0, Wikipedia Authors).
     #[derive(Debug, PartialEq, Eq, Clone, Copy)]
     pub enum Instruction {
-        /// Clear the display.
-        ClearDisplay = (0x0, 0x0, 0xE, 0x0),
+        /// Clear the screen.
+        ClearScreen = (0x0, 0x0, 0xE, 0x0),
         /// Return from a subroutine.
         Return = (0x0, 0x0, 0xE, 0xE),
         /// Calls the machine code subroutine at the `target_address`.
@@ -293,6 +293,8 @@ define_instruction! {
         ///
         /// If a carry occurs [`DataRegister::VF`] is set to `1`,
         /// if not it is set to `0`.
+        /// If the `target_register` is [`DataRegister::VF`],
+        /// the this will overwrite the result, discarding it.
         AddAssign {
             target_register: DataRegister,
             source_register: DataRegister,
@@ -303,6 +305,8 @@ define_instruction! {
         ///
         /// If a borrow occurs [`DataRegister::VF`] is set to `0`,
         /// if not it is set to `1`.
+        /// If the `target_register` is [`DataRegister::VF`],
+        /// the this will overwrite the result, discarding it.
         SubAssign {
             target_register: DataRegister,
             source_register: DataRegister,
@@ -312,6 +316,8 @@ define_instruction! {
         ///
         /// [`DataRegister::VF`] is set to the least significant bit of
         /// `source_register` prior to the shift, i.e. the bit that is shifted out.
+        /// If the `target_register` is [`DataRegister::VF`],
+        /// the this will overwrite the result, discarding it.
         ShrAssign {
             target_register: DataRegister,
             source_register: DataRegister,
@@ -322,6 +328,8 @@ define_instruction! {
         ///
         /// If a borrow occurs [`DataRegister::VF`] is set to `0`,
         /// if not it is set to `1`.
+        /// If the `target_register` is [`DataRegister::VF`],
+        /// the this will overwrite the result, discarding it.
         RevSubAssign {
             target_register: DataRegister,
             source_register: DataRegister,
@@ -331,6 +339,8 @@ define_instruction! {
         ///
         /// [`DataRegister::VF`] is set to the most significant bit of
         /// `source_register` prior to the shift, i.e. the bit that is shifted out.
+        /// If the `target_register` is [`DataRegister::VF`],
+        /// the this will overwrite the result, discarding it.
         ShlAssign {
             target_register: DataRegister,
             source_register: DataRegister,
@@ -352,16 +362,20 @@ define_instruction! {
         } = (0xC, target_register, mask),
         /// Draw a sprite at the position given by the values
         /// in `position_x_register` and `position_y_register`.
-        /// For this `last_sprite_byte_offset`+1 bytes of sprite data are read from the address
+        /// For this `sprite_byte_len` bytes of sprite data are read from the address
         /// stored in the special address register `I`.
         ///
+        /// The bits of sprite data are XORed onto the screen.
+        /// If a previously set bit of the screen is unset in the process,
+        /// [`DataRegister::VF`] is set to `1`, if not it is set to `0`.
+        ///
         /// If the position for the sprite to be drawn is offscreen,
         /// the position will have the modulo of the screen size in each dimension applied to it.
         DrawSprite {
             position_x_register: DataRegister,
             position_y_register: DataRegister,
-            last_sprite_byte_offset: U4,
-        } = (0xD, position_x_register, position_y_register, last_sprite_byte_offset),
+            sprite_byte_len: U4,
+        } = (0xD, position_x_register, position_y_register, sprite_byte_len),
         /// Skip the next instruction if the key corresponding
         /// to the value set in `key_register` is pressed.
         SkipIfKeyPressed { key_register: DataRegister } = (0xE, key_register, 0x9, 0xE),
@@ -373,15 +387,15 @@ define_instruction! {
         /// Wait until a key is pressed and store the value
         /// corresponding the key in `target_register`
         WaitForKeyPress { target_register: DataRegister } = (0xF, target_register, 0x0, 0xA),
-        /// Set the value of the delay timer to the value in `source_register`.
-        SetDelayTimer { source_register: DataRegister } = (0xF, source_register, 0x1, 0x5),
-        /// Set the value of the sound timer to the value in `source_register`.
-        SetSoundTimer { source_register: DataRegister } = (0xF, source_register, 0x1, 0x8),
+        /// Assign the value in `source_register` to the delay timer.
+        AssignToDelayTimer { source_register: DataRegister } = (0xF, source_register, 0x1, 0x5),
+        /// Assign the value in `source_register` to the sound timer.
+        AssignToSoundTimer { source_register: DataRegister } = (0xF, source_register, 0x1, 0x8),
         /// Add the value in `source_register` to the value
         /// in the special address register `I` and store the result in `I`.
         ///
         /// [`DataRegister::VF`] is not altered.
-        AddAssignI { source_register: DataRegister } = (0xF, source_register, 0x1, 0xE),
+        AddAssignToI { source_register: DataRegister } = (0xF, source_register, 0x1, 0xE),
         /// Assign the address of the built-in hex char sprite
         /// corresponding to the value in `sprite_register`
         /// to the special address register `I`.
@@ -452,7 +466,7 @@ mod test {
         let instr = Instruction::DrawSprite {
             position_x_register: DataRegister::V9,
             position_y_register: DataRegister::V3,
-            last_sprite_byte_offset: U4::try_from(5).unwrap(),
+            sprite_byte_len: U4::try_from(5).unwrap(),
         };
 
         let instr_bytes = [0xD9_u8, 0x35];
