@@ -1,7 +1,13 @@
-use std::{convert::TryFrom, time::Duration};
+use std::{
+    convert::TryFrom,
+    thread::{self, JoinHandle},
+    time::{Duration, Instant},
+};
 
+use flume::{Receiver, Sender};
 use rand::random;
 use thiserror::Error;
+use tracing::{debug, error, trace, trace_span, warn, Level};
 
 use crate::{
     builtin_sprites::font_4x5::Font,
@@ -138,6 +144,45 @@ impl PartialOffscreenDrawing {
     }
 }
 
+/// Skipping behavior for the processor.
+///
+/// Unsupported instructions that are not skipped will lead to an error being returned.
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum Skipping {
+    // Skip no unsupported instructions.
+    None,
+    // Skip [`Instruction::CallMachineSubroutine`].
+    CallMachineSubroutine,
+    // Skip unknown instructions
+    // (i.e. ones not specified as variants of [`Instruction`]).
+    Unknown,
+    // Skip all unsupported instructions.
+    // This skips both [`Instruction::CallMachineSubroutine`] and unknown ones.
+    AllUnsupported,
+}
+
+impl Default for Skipping {
+    fn default() -> Self {
+        Self::CallMachineSubroutine
+    }
+}
+
+impl Skipping {
+    pub fn should_skip_call_machine_subroutine(self) -> bool {
+        match self {
+            Self::CallMachineSubroutine | Self::AllUnsupported => true,
+            _ => false,
+        }
+    }
+
+    pub fn should_skip_unknown(self) -> bool {
+        match self {
+            Self::Unknown | Self::AllUnsupported => true,
+            _ => false,
+        }
+    }
+}
+
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum InstructionTiming {
     TotalTime { duration: Duration },
@@ -150,9 +195,34 @@ impl From<Duration> for InstructionTiming {
     }
 }
 
-// TODO: add screen and sprite handling
-// TODO: add delay and sound timer handling
-// TODO: add sound handling
+#[derive(Debug, PartialEq, Eq)]
+pub struct StepOutcome {
+    instruction_timing: InstructionTiming,
+    screen_updated: bool,
+    sound_timer_updated: bool,
+    delay_timer_updated: bool,
+}
+
+#[derive(Debug, PartialEq, Eq)]
+pub enum ProcessorEvent {
+    ScreenUpdate {
+        new_screen:
+            [u8; Processor::SCREEN_WIDTH_BYTES as usize * Processor::SCREEN_HEIGHT as usize],
+    },
+    SoundRequest {
+        duration: Duration,
+    },
+    WaitForKeyPress,
+    ErrorEncountered {
+        error: ProcessorError,
+    },
+}
+
+#[derive(Debug, PartialEq, Eq)]
+pub enum ControlEvent {
+    KeyStateChange { key: Key, new_state: KeyState },
+}
+
 // TODO: consider supporting S-CHIP as well
 #[derive(Debug, PartialEq, Eq)]
 pub struct Processor {
@@ -167,7 +237,7 @@ pub struct Processor {
     key_states: [KeyState; std::mem::variant_count::<Key>()],
     waiting_for_keypress: KeyWaitingState,
     partial_offscreen_drawing: PartialOffscreenDrawing,
-    skip_call_machine_subroutine: bool,
+    skipping: Skipping,
 }
 
 impl Default for Processor {
@@ -188,7 +258,7 @@ impl Processor {
     /// don't allow access outside of this and will cause a runtime error when run
     /// with the sum of the address in special address register `I`
     /// and the number of the `last_register` greater than this.
-    /// The same goes for [`Instruction::DrawSprite`] and the `last_sprite_byte_offset`.
+    /// The same goes for [`Instruction::DrawSprite`] and the `sprite_byte_len`.
     pub const MAX_ADDRESS: u16 = u16::MAX;
 
     /// Screen width in bytes.
@@ -202,6 +272,139 @@ impl Processor {
         ProcessorBuilder::new()
     }
 
+    // TODO: add option for time scaling
+    #[tracing::instrument(skip(self, processor_event_sender, control_event_receiver))]
+    fn run(
+        mut self,
+        processor_event_sender: Sender<ProcessorEvent>,
+        control_event_receiver: Receiver<ControlEvent>,
+    ) -> Result<Processor, ProcessorError> {
+        let mut delay_duration = Duration::from_secs(0);
+        let mut last_delay_update = Instant::now();
+        loop {
+            let span = trace_span!("run loop iteration");
+            let _guard = span.enter();
+
+            let start = Instant::now();
+
+            let control_event = match control_event_receiver.try_recv() {
+                Ok(control_event) => Some(control_event),
+                Err(flume::TryRecvError::Empty) => None,
+                Err(flume::TryRecvError::Disconnected) => {
+                    debug!("control_event_sender dropped, stopping processor::run");
+                    break;
+                } // ProcessorHandle dropped, stop
+            };
+            if let Some(control_event) = control_event {
+                match control_event {
+                    ControlEvent::KeyStateChange { key, new_state } => {
+                        self.set_key_state(key, new_state)
+                    }
+                }
+            }
+
+            self.delay_timer = (delay_duration
+                .saturating_sub(last_delay_update.elapsed())
+                .as_secs_f64()
+                * 60.0) as u8;
+
+            let outcome = match self.step() {
+                Ok(outcome) => outcome,
+                Err(error) => {
+                    error!(?error, "error stepping processor");
+                    if let Err(_) =
+                        processor_event_sender.send(ProcessorEvent::ErrorEncountered { error })
+                    {
+                        debug!("processor_event_receiver dropped, stopping processor::run");
+                    }
+                    break;
+                }
+            };
+
+            if outcome.screen_updated {
+                if let Err(_) = processor_event_sender.send(ProcessorEvent::ScreenUpdate {
+                    new_screen: self.screen.clone(),
+                }) {
+                    debug!("processor_event_receiver dropped, stopping processor::run");
+                    break;
+                }
+            }
+
+            if outcome.sound_timer_updated {
+                if let Err(_) = processor_event_sender.send(ProcessorEvent::SoundRequest {
+                    duration: Duration::from_secs_f64(self.sound_timer as f64 / 60.0),
+                }) {
+                    debug!("processor_event_receiver dropped, stopping processor::run");
+                    break;
+                }
+            }
+
+            if outcome.delay_timer_updated {
+                delay_duration = Duration::from_secs_f64(self.delay_timer as f64 / 60.0); // / 500;
+                last_delay_update = Instant::now();
+            }
+
+            match outcome.instruction_timing {
+                InstructionTiming::TotalTime { duration } => {
+                    // let duration = duration / 500;
+
+                    let elapsed = start.elapsed();
+                    let delta = duration.checked_sub(elapsed);
+                    if let Some(delta) = delta {
+                        spin_sleep::sleep(delta);
+                    } else {
+                        warn!(
+                            target_duration = ?duration,
+                            actual = ?elapsed,
+                            delta = ?(elapsed - duration),
+                            "instruction processing took longer than target duration",
+                        );
+                    }
+                }
+                InstructionTiming::WaitForKeyPress => {
+                    if let Err(_) = processor_event_sender.send(ProcessorEvent::WaitForKeyPress) {
+                        debug!("processor_event_receiver dropped, stopping processor::run");
+                        break;
+                    }
+
+                    let control_event = match control_event_receiver.recv() {
+                        Ok(control_event) => Some(control_event),
+                        Err(flume::RecvError::Disconnected) => break, // ProcessorHandle dropped, stop
+                    };
+                    if let Some(control_event) = control_event {
+                        match control_event {
+                            ControlEvent::KeyStateChange { key, new_state } => {
+                                self.set_key_state(key, new_state)
+                            }
+                        }
+                    }
+                }
+            }
+        }
+
+        Ok(self)
+    }
+
+    // TODO: add tests for this
+    #[tracing::instrument(skip(self))]
+    pub fn start(
+        self,
+    ) -> (
+        Sender<ControlEvent>,
+        Receiver<ProcessorEvent>,
+        JoinHandle<Result<Processor, ProcessorError>>,
+    ) {
+        let (processor_event_sender, processor_event_receiver) = flume::bounded(2);
+        let (control_event_sender, control_event_receiver) = flume::bounded(2);
+
+        let join_handle = thread::Builder::new()
+            .name("processor runner".to_owned())
+            .spawn(move || self.run(processor_event_sender, control_event_receiver))
+            .expect("failed to create thread for processor runner");
+
+        (control_event_sender, processor_event_receiver, join_handle)
+    }
+
     /// Get the value of a data register.
     const fn get_register(&self, register: DataRegister) -> u8 {
         self.data_registers[register as u8 as usize]
@@ -219,6 +422,7 @@ impl Processor {
 
     /// Set the state of a key.
     pub fn set_key_state(&mut self, key: Key, state: KeyState) {
+        debug!(?key, ?state, "key state update");
         // TODO: possibly replace with if let chain once let_chains (eRFC 2497) work
         match self.waiting_for_keypress {
             KeyWaitingState::Waiting { target_register } if state == KeyState::Pressed => {
@@ -230,6 +434,12 @@ impl Processor {
         self.key_states[key as u8 as usize] = state;
     }
 
+    pub const fn screen(
+        &self,
+    ) -> &[u8; Self::SCREEN_WIDTH_BYTES as usize * Self::SCREEN_HEIGHT as usize] {
+        &self.screen
+    }
+
     /// Draw byte to `screen` at position `byte_x*8` and `y`.
     /// Does not take `&mut self` so it can be called while iterating over a sprite in `self.memory`.
     ///
@@ -258,7 +468,8 @@ impl Processor {
         [num / 100, num / 10 % 10, num % 10]
     }
 
-    pub fn step(&mut self) -> Result<InstructionTiming, ProcessorError> {
+    #[tracing::instrument(level = Level::TRACE, skip(self))]
+    pub fn step(&mut self) -> Result<StepOutcome, ProcessorError> {
         if self.program_counter >= Self::MAX_ADDRESS {
             return Err(ProcessorError::OutOfBoundsMemoryAccess {
                 program_counter: self.program_counter,
@@ -272,12 +483,39 @@ impl Processor {
             self.memory[self.program_counter as usize + 1],
         ];
 
-        let instruction = Instruction::try_from(instruction_nibbles)?;
+        let instruction = match Instruction::try_from(instruction_nibbles) {
+            Ok(instruction) => instruction,
+            Err(error) => {
+                if !self.skipping.should_skip_unknown() {
+                    return Err(error.into());
+                } else {
+                    self.program_counter = self
+                        .program_counter
+                        .wrapping_add(std::mem::size_of::<u16>() as u16);
+
+                    return Ok(StepOutcome {
+                        instruction_timing: Duration::from_micros(0).into(),
+                        screen_updated: false,
+                        sound_timer_updated: false,
+                        delay_timer_updated: false,
+                    });
+                }
+            }
+        };
+
+        trace!(program_counter = ?self.program_counter, ?instruction, "executing instruction");
 
         let mut was_control_flow_instr = false;
+        let mut screen_updated = false;
+        let mut sound_timer_updated = false;
+        let mut delay_timer_updated = false;
 
         match instruction {
-            Instruction::ClearDisplay => self.screen.fill(0),
+            Instruction::ClearScreen => {
+                self.screen.fill(0);
+
+                screen_updated = true;
+            }
             Instruction::Return => {
                 self.program_counter =
                     self.call_stack
@@ -289,7 +527,7 @@ impl Processor {
                 was_control_flow_instr = true;
             }
             Instruction::CallMachineSubroutine { .. } => {
-                if !self.skip_call_machine_subroutine {
+                if !self.skipping.should_skip_call_machine_subroutine() {
                     return Err(ProcessorError::CallMachineSubroutineUnsupported {
                         program_counter: self.program_counter,
                     });
@@ -386,8 +624,8 @@ impl Processor {
                 let (res, carry) = self
                     .get_register(target_register)
                     .overflowing_add(self.get_register(source_register));
-                self.set_register(DataRegister::VF, carry as u8);
                 self.set_register(target_register, res);
+                self.set_register(DataRegister::VF, carry as u8);
             }
             Instruction::SubAssign {
                 target_register,
@@ -396,16 +634,16 @@ impl Processor {
                 let (res, borrow) = self
                     .get_register(target_register)
                     .overflowing_sub(self.get_register(source_register));
-                self.set_register(DataRegister::VF, 1 - borrow as u8);
                 self.set_register(target_register, res);
+                self.set_register(DataRegister::VF, 1 - borrow as u8);
             }
             Instruction::ShrAssign {
                 target_register,
                 source_register,
             } => {
                 let old_lsb = self.get_register(source_register) & 0b1;
-                self.set_register(DataRegister::VF, old_lsb);
                 self.set_register(target_register, self.get_register(source_register) >> 1);
+                self.set_register(DataRegister::VF, old_lsb);
             }
             Instruction::RevSubAssign {
                 target_register,
@@ -414,16 +652,16 @@ impl Processor {
                 let (res, borrow) = self
                     .get_register(source_register)
                     .overflowing_sub(self.get_register(target_register));
-                self.set_register(DataRegister::VF, 1 - borrow as u8);
                 self.set_register(target_register, res);
+                self.set_register(DataRegister::VF, 1 - borrow as u8);
             }
             Instruction::ShlAssign {
                 target_register,
                 source_register,
             } => {
                 let old_msb = (self.get_register(source_register) >> 7) & 0b1;
-                self.set_register(DataRegister::VF, old_msb);
                 self.set_register(target_register, self.get_register(source_register) << 1);
+                self.set_register(DataRegister::VF, old_msb);
             }
             Instruction::SkipIfNeq {
                 register1,
@@ -454,10 +692,9 @@ impl Processor {
             Instruction::DrawSprite {
                 position_x_register,
                 position_y_register,
-                last_sprite_byte_offset,
+                sprite_byte_len,
             } => {
-                if self.address_register
-                    > Self::MAX_ADDRESS - u8::from(last_sprite_byte_offset) as u16
+                if self.address_register > Self::MAX_ADDRESS - u8::from(sprite_byte_len) as u16 + 1
                 {
                     return Err(ProcessorError::OutOfBoundsMemoryAccess {
                         program_counter: self.program_counter,
@@ -468,8 +705,7 @@ impl Processor {
                 let mut set_pixel_unset = false;
 
                 for (i, sprite_byte) in self.memory[self.address_register as usize
-                    ..=(self.address_register as usize
-                        + u8::from(last_sprite_byte_offset) as usize)]
+                    ..(self.address_register as usize + u8::from(sprite_byte_len) as usize)]
                     .iter()
                     .copied()
                     .enumerate()
@@ -510,6 +746,8 @@ impl Processor {
                 }
 
                 self.set_register(DataRegister::VF, set_pixel_unset as u8);
+
+                screen_updated = true;
             }
             Instruction::SkipIfKeyPressed { key_register } => {
                 let key_id = self.get_register(key_register);
@@ -550,15 +788,20 @@ impl Processor {
             }
             Instruction::WaitForKeyPress { target_register } => {
                 self.waiting_for_keypress = KeyWaitingState::Waiting { target_register };
-                todo!()
+                // actual waiting is done by the caller of step()
+                // and is signaled by the instruction_timing of the StepOutcome
             }
-            Instruction::SetDelayTimer { source_register } => {
-                self.delay_timer = self.get_register(source_register)
+            Instruction::AssignToDelayTimer { source_register } => {
+                self.delay_timer = self.get_register(source_register);
+
+                delay_timer_updated = true;
             }
-            Instruction::SetSoundTimer { source_register } => {
-                self.sound_timer = self.get_register(source_register)
+            Instruction::AssignToSoundTimer { source_register } => {
+                self.sound_timer = self.get_register(source_register);
+
+                sound_timer_updated = true;
             }
-            Instruction::AddAssignI { source_register } => {
+            Instruction::AddAssignToI { source_register } => {
                 self.address_register = self
                     .address_register
                     .wrapping_add(self.get_register(source_register) as u16)
@@ -623,7 +866,12 @@ impl Processor {
                 .wrapping_add(std::mem::size_of::<u16>() as u16);
         }
 
-        Ok(self.instruction_timing(instruction))
+        Ok(StepOutcome {
+            instruction_timing: self.instruction_timing(instruction),
+            screen_updated,
+            sound_timer_updated,
+            delay_timer_updated,
+        })
     }
 
     /// Return the average total amount of time an instruction should take,
@@ -642,7 +890,7 @@ impl Processor {
     fn instruction_timing(&self, instruction: Instruction) -> InstructionTiming {
         match instruction {
             Instruction::CallMachineSubroutine { .. } => Duration::from_micros(0).into(), // unsupported
-            Instruction::ClearDisplay => Duration::from_micros(109).into(),
+            Instruction::ClearScreen => Duration::from_micros(109).into(),
             Instruction::Return
             | Instruction::Jump { .. }
             | Instruction::CallSubroutine { .. }
@@ -712,8 +960,8 @@ impl Processor {
             Instruction::AssignConst { .. } => Duration::from_micros(27).into(),
             Instruction::AddAssignConst { .. }
             | Instruction::AssignDelayTimerVal { .. }
-            | Instruction::SetDelayTimer { .. }
-            | Instruction::SetSoundTimer { .. } => Duration::from_micros(45).into(),
+            | Instruction::AssignToDelayTimer { .. }
+            | Instruction::AssignToSoundTimer { .. } => Duration::from_micros(45).into(),
             Instruction::Assign { .. }
             | Instruction::OrAssign { .. }
             | Instruction::AndAssign { .. }
@@ -727,7 +975,7 @@ impl Processor {
             Instruction::AssignRandomMasked { .. } => Duration::from_micros(164).into(),
             Instruction::DrawSprite { .. } => Duration::from_micros(22_734).into(), // ignores intricacies, simply the average time
             Instruction::WaitForKeyPress { .. } => InstructionTiming::WaitForKeyPress,
-            Instruction::AddAssignI { .. } => Duration::from_micros(86).into(), // ignores page boundaries of original chip
+            Instruction::AddAssignToI { .. } => Duration::from_micros(86).into(), // ignores page boundaries of original chip
             Instruction::AssignHexCharSpriteAddrToI { .. } => Duration::from_micros(91).into(),
             Instruction::StoreBCD { source_register } => {
                 let val = self.get_register(source_register);
@@ -773,7 +1021,7 @@ impl ProcessorBuilder {
                 key_states: [KeyState::default(); 16],
                 waiting_for_keypress: KeyWaitingState::default(),
                 partial_offscreen_drawing: PartialOffscreenDrawing::default(),
-                skip_call_machine_subroutine: true,
+                skipping: Skipping::default(),
             },
             font: Font::default(),
         }
@@ -781,18 +1029,15 @@ impl ProcessorBuilder {
 
     // TODO: consider taking an iterator as input
     /// Copies the program into the processors memory
-    /// in such that the first element is at address 0x0.
-    /// This way the index of bytes in the program represents the address in memory.
-    /// IMPORTANT: The first [`Font::LEN`] elements are overwritten with sprite font data
-    ///            and are therefore not usable for program data.
+    /// in such that the first byte is at address 0x200.
     pub fn program(mut self, program: &[u8]) -> Result<Self, ProcessorBuilderError> {
-        if program.len() > Processor::MAX_USABLE_MEMORY_LEN {
+        if program.len() > Processor::MAX_USABLE_MEMORY_LEN - 0x200 {
             return Err(ProcessorBuilderError::ProgramExceedsUsableMemoryLen {
                 program_len: program.len(),
             });
         }
 
-        self.processor.memory[Font::LEN..program.len()].copy_from_slice(program);
+        self.processor.memory[0x200..(0x200 + program.len())].copy_from_slice(&program);
 
         Ok(self)
     }
@@ -815,10 +1060,11 @@ impl ProcessorBuilder {
         self
     }
 
-    /// Make the processor skip [`Instruction::CallMachineSubroutine`] instructions
-    /// instead of returning an error.
-    pub fn skip_call_machine_subroutine(mut self) -> Self {
-        self.processor.skip_call_machine_subroutine = true;
+    /// Set the processor's skipping behavior of unsupported instructions.
+    /// Unsupported instructions that are not skipped cause an error to be returned.
+    /// See also [`Skipping`].
+    pub fn skipping(mut self, skipping: Skipping) -> Self {
+        self.processor.skipping = skipping;
         self
     }
 
