@@ -17,7 +17,7 @@ mod step {
             processor.step(),
             Err(ProcessorError::OutOfBoundsMemoryAccess {
                 program_counter: (Processor::MAX_USABLE_MEMORY_LEN - 1) as u16
-            }) as Result<InstructionTiming, _>
+            }) as Result<StepOutcome, _>
         );
     }
 
@@ -33,6 +33,57 @@ mod step {
         };
     }
 
+    mod instr_unknown {
+        use super::*;
+
+        #[test]
+        fn case_skip() {
+            let mut program = [0; Processor::MAX_USABLE_MEMORY_LEN];
+            let instruction_bytes = [0xFF, 0xFF];
+            program[0x200..=0x201].copy_from_slice(&instruction_bytes);
+
+            let mut processor = Processor {
+                memory: program.clone(),
+                program_counter: 0x200,
+                skipping: Skipping::Unknown,
+                ..Processor::default()
+            };
+
+            processor.step().unwrap();
+
+            assert_eq!(
+                processor,
+                Processor {
+                    memory: program,
+                    program_counter: 0x202,
+                    skipping: Skipping::Unknown,
+                    ..Processor::default()
+                }
+            );
+        }
+
+        #[test]
+        fn case_err() {
+            let mut program = [0; Processor::MAX_USABLE_MEMORY_LEN];
+            let instruction_bytes = [0xFF, 0xFF];
+            program[0x200..=0x201].copy_from_slice(&instruction_bytes);
+
+            let mut processor = Processor {
+                memory: program.clone(),
+                program_counter: 0x200,
+                skipping: Skipping::None,
+                ..Processor::default()
+            };
+
+            assert_eq!(
+                processor.step(),
+                Err(ProcessorError::InvalidInstructionNibblesError(
+                    Instruction::try_from([0xFF, 0xFF]).unwrap_err()
+                )) as Result<StepOutcome, _>
+            );
+        }
+    }
+
     mod instr_call_machine_subroutine {
         use super::*;
 
@@ -47,7 +98,7 @@ mod step {
             let mut processor = Processor {
                 memory: program.clone(),
                 program_counter: 0x200,
-                skip_call_machine_subroutine: true,
+                skipping: Skipping::CallMachineSubroutine,
                 ..Processor::default()
             };
 
@@ -58,7 +109,7 @@ mod step {
                 Processor {
                     memory: program,
                     program_counter: 0x202,
-                    skip_call_machine_subroutine: true,
+                    skipping: Skipping::CallMachineSubroutine,
                     ..Processor::default()
                 }
             );
@@ -75,7 +126,7 @@ mod step {
             let mut processor = Processor {
                 memory: program.clone(),
                 program_counter: 0x200,
-                skip_call_machine_subroutine: false,
+                skipping: Skipping::None,
                 ..Processor::default()
             };
 
@@ -83,15 +134,15 @@ mod step {
                 processor.step(),
                 Err(ProcessorError::CallMachineSubroutineUnsupported {
                     program_counter: 0x200
-                }) as Result<InstructionTiming, _>
+                }) as Result<StepOutcome, _>
             );
         }
     }
 
     #[test]
-    fn instr_clear_display() {
+    fn instr_clear_screen() {
         let mut program = [0; Processor::MAX_USABLE_MEMORY_LEN];
-        let instruction_bytes = <[u8; 2]>::from(Instruction::ClearDisplay);
+        let instruction_bytes = <[u8; 2]>::from(Instruction::ClearScreen);
         program[0x200..=0x201].copy_from_slice(&instruction_bytes);
 
         let mut screen =
@@ -164,7 +215,7 @@ mod step {
                 processor.step(),
                 Err(ProcessorError::ReturnWithEmptyCallStack {
                     program_counter: 0x204
-                }) as Result<InstructionTiming, _>
+                }) as Result<StepOutcome, _>
             );
         }
     }
@@ -241,7 +292,7 @@ mod step {
                 processor.step(),
                 Err(ProcessorError::MaxCallStackSizeExceeded {
                     program_counter: 0x200,
-                }) as Result<InstructionTiming, _>,
+                }) as Result<StepOutcome, _>,
             );
         }
     }
@@ -479,23 +530,25 @@ mod step {
             (
                 $test_name:ident,
                 $instr_name:ident,
-                target_val: $target_val:expr,
-                source_val: $source_val:expr,
+                target: ($target_register:ident, $target_val:expr),
+                source: ($source_register:ident, $source_val:expr),
                 result: $result:expr,
                 vf: $vf:literal
             ) => {
                 #[test]
                 fn $test_name() {
+                    use DataRegister::*;
+
                     let mut program = [0; Processor::MAX_USABLE_MEMORY_LEN];
                     let instruction_bytes = <[u8; 2]>::from(Instruction::$instr_name {
-                        target_register: DataRegister::V3,
-                        source_register: DataRegister::V9,
+                        target_register: $target_register,
+                        source_register: $source_register,
                     });
                     program[0x200..=0x201].copy_from_slice(&instruction_bytes);
 
                     let mut data_registers = [0; 16];
-                    data_registers[DataRegister::V3 as u8 as usize] = $target_val;
-                    data_registers[DataRegister::V9 as u8 as usize] = $source_val;
+                    data_registers[$target_register as u8 as usize] = $target_val;
+                    data_registers[$source_register as u8 as usize] = $source_val;
 
                     let mut processor = Processor {
                         data_registers,
@@ -506,9 +559,9 @@ mod step {
                     processor.step().unwrap();
 
                     let mut expected_data_registers = [0; 16];
-                    expected_data_registers[DataRegister::V3 as u8 as usize] = $result;
-                    expected_data_registers[DataRegister::V9 as u8 as usize] = $source_val;
-                    expected_data_registers[DataRegister::VF as u8 as usize] = $vf;
+                    expected_data_registers[$target_register as u8 as usize] = $result;
+                    expected_data_registers[$source_register as u8 as usize] = $source_val;
+                    expected_data_registers[VF as u8 as usize] = $vf;
 
                     assert_eq!(
                         processor,
@@ -523,43 +576,48 @@ mod step {
             };
         }
 
-        generate_test!(or, OrAssign, target_val: 0b10101010, source_val: 0b11001010, result: 0b11101010, vf: 0);
-        generate_test!(and, AndAssign, target_val: 0b10101010, source_val: 0b11001010, result: 0b10001010, vf: 0);
-        generate_test!(xor, XorAssign, target_val: 0b10101010, source_val: 0b11001010, result: 0b01100000, vf: 0);
+        generate_test!(or, OrAssign, target: (V3, 0b10101010), source: (V9, 0b11001010), result: 0b11101010, vf: 0);
+        generate_test!(and, AndAssign, target: (V3, 0b10101010), source: (V9, 0b11001010), result: 0b10001010, vf: 0);
+        generate_test!(xor, XorAssign, target: (V3, 0b10101010), source: (V9, 0b11001010), result: 0b01100000, vf: 0);
 
         mod add {
             use super::*;
 
-            generate_test!(case_carry, AddAssign, target_val: !0, source_val: 1, result: 0, vf: 1);
-            generate_test!(case_no_carry, AddAssign, target_val: 3, source_val: 4, result: 7, vf: 0);
+            generate_test!(case_carry, AddAssign, target: (V3, !0), source: (V9, 1), result: 0, vf: 1);
+            generate_test!(case_no_carry, AddAssign, target: (V3, 3), source: (V9, 4), result: 7, vf: 0);
+            generate_test!(case_target_is_vf, AddAssign, target: (VF, 3), source: (V7, 4), result: 0, vf: 0);
         }
 
         mod sub {
             use super::*;
 
-            generate_test!(case_borrow, SubAssign, target_val: 0, source_val: 1, result: !0, vf: 0);
-            generate_test!(case_no_borrow, SubAssign, target_val: 7, source_val: 3, result: 4, vf: 1);
+            generate_test!(case_borrow, SubAssign, target: (V3, 0), source: (V9, 1), result: !0, vf: 0);
+            generate_test!(case_no_borrow, SubAssign, target: (V3, 7), source: (V9, 3), result: 4, vf: 1);
+            generate_test!(case_target_is_vf, SubAssign, target: (VF, 7), source: (V7, 3), result: 1, vf: 1);
         }
 
         mod rev_sub {
             use super::*;
 
-            generate_test!(case_borrow, RevSubAssign, target_val: 1, source_val: 0, result: !0, vf: 0);
-            generate_test!(case_no_borrow, RevSubAssign, target_val: 3, source_val: 7, result: 4, vf: 1);
+            generate_test!(case_borrow, RevSubAssign, target: (V3, 1), source: (V9, 0), result: !0, vf: 0);
+            generate_test!(case_no_borrow, RevSubAssign, target: (V3, 3), source: (V9, 7), result: 4, vf: 1);
+            generate_test!(case_target_is_vf, RevSubAssign, target: (VF, 3), source: (V7, 7), result: 1, vf: 1);
         }
 
         mod shr {
             use super::*;
 
-            generate_test!(case_old_lsb_set, ShrAssign, target_val: 0b111, source_val: 0b101, result: 0b10, vf: 1);
-            generate_test!(case_old_lsb_unset, ShrAssign, target_val: 0b111, source_val: 0b100, result: 0b10, vf: 0);
+            generate_test!(case_old_lsb_set, ShrAssign, target: (V3, 0b111), source: (V9, 0b101), result: 0b10, vf: 1);
+            generate_test!(case_old_lsb_unset, ShrAssign, target: (V3, 0b111), source: (V9, 0b100), result: 0b10, vf: 0);
+            generate_test!(case_target_is_vf, ShrAssign, target: (VF, 0b111), source: (V7, 0b100), result: 0, vf: 0);
         }
 
         mod shl {
             use super::*;
 
-            generate_test!(case_old_msb_set, ShlAssign, target_val: 0b1110_0000, source_val: 0b1010_0000, result: 0b0100_0000, vf: 1);
-            generate_test!(case_old_msb_unset, ShlAssign, target_val: 0b1110_0000, source_val: 0b0010_0000, result: 0b0100_0000, vf: 0);
+            generate_test!(case_old_msb_set, ShlAssign, target: (V3, 0b1110_0000), source: (V9, 0b1010_0000), result: 0b0100_0000, vf: 1);
+            generate_test!(case_old_msb_unset, ShlAssign, target: (V3, 0b1110_0000), source: (V9, 0b0010_0000), result: 0b0100_0000, vf: 0);
+            generate_test!(case_target_is_vf, ShlAssign, target: (VF, 0b1110_0000), source: (V7, 0b0010_0000), result: 0, vf: 0);
         }
     }
 
@@ -679,7 +737,7 @@ mod step {
             let instruction_bytes = <[u8; 2]>::from(Instruction::DrawSprite {
                 position_x_register: DataRegister::V0,
                 position_y_register: DataRegister::V1,
-                last_sprite_byte_offset: U4::try_from(7).unwrap(), // 8x8 sprite
+                sprite_byte_len: U4::try_from(8).unwrap(), // 8x8 sprite
             });
             program[0x200..=0x201].copy_from_slice(&instruction_bytes);
             program[0x300..=0x307].copy_from_slice(&SPRITE_BYTES);
@@ -758,7 +816,7 @@ mod step {
             let instruction_bytes = <[u8; 2]>::from(Instruction::DrawSprite {
                 position_x_register: DataRegister::V0,
                 position_y_register: DataRegister::V1,
-                last_sprite_byte_offset: U4::try_from(7).unwrap(), // 8x8 sprite
+                sprite_byte_len: U4::try_from(8).unwrap(), // 8x8 sprite
             });
             program[0x200..=0x201].copy_from_slice(&instruction_bytes);
             program[0x300..=0x307].copy_from_slice(&SPRITE_BYTES);
@@ -837,7 +895,7 @@ mod step {
             let instruction_bytes = <[u8; 2]>::from(Instruction::DrawSprite {
                 position_x_register: DataRegister::V0,
                 position_y_register: DataRegister::V1,
-                last_sprite_byte_offset: U4::try_from(7).unwrap(), // 8x8 sprite
+                sprite_byte_len: U4::try_from(8).unwrap(), // 8x8 sprite
             });
             program[0x200..=0x201].copy_from_slice(&instruction_bytes);
             program[0x300..=0x307].copy_from_slice(&SPRITE_BYTES);
@@ -916,7 +974,7 @@ mod step {
             let instruction_bytes = <[u8; 2]>::from(Instruction::DrawSprite {
                 position_x_register: DataRegister::V0,
                 position_y_register: DataRegister::V1,
-                last_sprite_byte_offset: U4::try_from(7).unwrap(), // 8x8 sprite
+                sprite_byte_len: U4::try_from(8).unwrap(), // 8x8 sprite
             });
             program[0x200..=0x201].copy_from_slice(&instruction_bytes);
             program[0x300..=0x307].copy_from_slice(&SPRITE_BYTES);
@@ -995,7 +1053,7 @@ mod step {
             let instruction_bytes = <[u8; 2]>::from(Instruction::DrawSprite {
                 position_x_register: DataRegister::V0,
                 position_y_register: DataRegister::V1,
-                last_sprite_byte_offset: U4::try_from(1).unwrap(),
+                sprite_byte_len: U4::try_from(2).unwrap(),
             });
             program[0x200..=0x201].copy_from_slice(&instruction_bytes);
 
@@ -1012,7 +1070,7 @@ mod step {
                 processor.step(),
                 Err(ProcessorError::OutOfBoundsMemoryAccess {
                     program_counter: 0x200
-                }) as Result<InstructionTiming, _>
+                }) as Result<StepOutcome, _>
             );
         }
     }
@@ -1072,7 +1130,7 @@ mod step {
                         let mut processor = Processor {
                             data_registers,
                             memory: program.clone(),
-                            key_states: key_states,
+                            key_states,
                             ..Processor::default()
                         };
 
@@ -1084,7 +1142,7 @@ mod step {
                                 data_registers,
                                 memory: program,
                                 program_counter: if $negated { 0x202 } else { 0x204 },
-                                key_states: key_states,
+                                key_states,
                                 ..Processor::default()
                             }
                         );
@@ -1155,9 +1213,9 @@ mod step {
     }
 
     #[test]
-    fn instr_set_delay_timer() {
+    fn instr_assign_to_delay_timer() {
         let mut program = [0; Processor::MAX_USABLE_MEMORY_LEN];
-        let instruction_bytes = <[u8; 2]>::from(Instruction::SetDelayTimer {
+        let instruction_bytes = <[u8; 2]>::from(Instruction::AssignToDelayTimer {
             source_register: DataRegister::V8,
         });
         program[0x200..=0x201].copy_from_slice(&instruction_bytes);
@@ -1186,9 +1244,9 @@ mod step {
     }
 
     #[test]
-    fn instr_set_sound_timer() {
+    fn instr_assign_to_sound_timer() {
         let mut program = [0; Processor::MAX_USABLE_MEMORY_LEN];
-        let instruction_bytes = <[u8; 2]>::from(Instruction::SetSoundTimer {
+        let instruction_bytes = <[u8; 2]>::from(Instruction::AssignToSoundTimer {
             source_register: DataRegister::V8,
         });
         program[0x200..=0x201].copy_from_slice(&instruction_bytes);
@@ -1216,13 +1274,13 @@ mod step {
         );
     }
 
-    mod instr_add_assign_i {
+    mod instr_add_assign_to_i {
         use super::*;
 
         #[test]
         fn case_carry() {
             let mut program = [0; Processor::MAX_USABLE_MEMORY_LEN];
-            let instruction_bytes = <[u8; 2]>::from(Instruction::AddAssignI {
+            let instruction_bytes = <[u8; 2]>::from(Instruction::AddAssignToI {
                 source_register: DataRegister::V0,
             });
             program[0x200..=0x201].copy_from_slice(&instruction_bytes);
@@ -1256,7 +1314,7 @@ mod step {
         #[test]
         fn case_no_carry() {
             let mut program = [0; Processor::MAX_USABLE_MEMORY_LEN];
-            let instruction_bytes = <[u8; 2]>::from(Instruction::AddAssignI {
+            let instruction_bytes = <[u8; 2]>::from(Instruction::AddAssignToI {
                 source_register: DataRegister::V0,
             });
             program[0x200..=0x201].copy_from_slice(&instruction_bytes);
@@ -1483,7 +1541,7 @@ mod step {
                 processor.step(),
                 Err(ProcessorError::OutOfBoundsMemoryAccess {
                     program_counter: 0x200
-                }) as Result<InstructionTiming, _>
+                }) as Result<StepOutcome, _>
             );
         }
     }
@@ -1551,7 +1609,7 @@ mod step {
                 processor.step(),
                 Err(ProcessorError::OutOfBoundsMemoryAccess {
                     program_counter: 0x200
-                }) as Result<InstructionTiming, _>
+                }) as Result<StepOutcome, _>
             );
         }
     }
